@@ -131,7 +131,7 @@
 #' is estimated via ordinary least squares.}
 #'
 #' \item{If \code{name == "L1-L2"}, then
-#' \deqn{||\mathbf{w}||_1 = 1,\:\:\: ||\mathbf{w}||_2 \leq Q,}
+#' \deqn{||\mathbf{w}||_1 = 1,\:\:\: ||\mathbf{w}||_2 \leq Q, \:\:\: w_j \geq 0,\:\: j =1,\ldots,J.}
 #' where \eqn{Q} is a tuning parameter computed as in the "ridge" case.}
 #' }}
 #'
@@ -269,7 +269,7 @@ scest <- function(data,
 
   # Create weighting matrix
   if (is.character(V) == FALSE) {
-    stop("The object V should be a string! If you want to input a matrix use the option V.mat!")
+    stop("The object V should be a string! If you want to specify manually the weighting matrix use the option V.mat!")
   }
   
   if (is.null(V.mat) == FALSE) {
