@@ -34,7 +34,7 @@
 #' \deqn{(\mathbf{A}-\mathbf{B}\mathbf{w}-\mathbf{C}\mathbf{r})'\mathbf{V}(\mathbf{A}-\mathbf{B}\mathbf{w}-\mathbf{C}\mathbf{r})}
 #' The default is the identity matrix, so equal weight is given to all observations. In the case of multiple treated observations
 #' (you used \code{\link{scdataMulti}} to prepare the data), the user can specify \code{V} as a string equal to either "separate" or "pooled".
-#' If \code{scdata()} was used to prepare the data, \code{V} is automatically set to "separate" as the two options are 
+#' If \code{scdata()} was used to prepare the data, \code{V} is automatically set to "separate" as the two options are
 #' equivalent. See the \strong{Details} section for more.
 #' @param V.mat A conformable weighting matrix \eqn{\mathbf{V}} to be used in the minimization of the sum of squared residuals
 #' \deqn{(\mathbf{A}-\mathbf{B}\mathbf{w}-\mathbf{C}\mathbf{r})'\mathbf{V}(\mathbf{A}-\mathbf{B}\mathbf{w}-\mathbf{C}\mathbf{r}).}
@@ -66,7 +66,7 @@
 #'  "gaussian" which uses conditional subgaussian bounds; "ls" which specifies a location-scale model for \eqn{\mathbf{u}}; "qreg" which employs a
 #'  quantile regressions to get the conditional bounds; "all" uses each one of the previous methods.
 #' @param e.order a scalar that sets the order of the polynomial in \eqn{\mathbf{B}} when predicting moments of \eqn{\mathbf{e}}.
-#' The default is \code{e.order = 1}, however if there is risk of over-fitting, the command automatically sets it 
+#' The default is \code{e.order = 1}, however if there is risk of over-fitting, the command automatically sets it
 #' to \code{e.order = 0}. See the \strong{Details} section for more information.
 #' @param e.lags a scalar that sets the number of lags of \eqn{\mathbf{B}} when predicting moments of \eqn{\mathbf{e}}.
 #' The default is \code{e.order = 1}, however if there is risk of over-fitting, the command automatically sets it
@@ -78,7 +78,8 @@
 #' @param plot a logical specifying whether \code{\link{scplot}} should be called and a plot saved in the current working
 #' directory. For more options see \code{\link{scplot}}.
 #' @param plot.name a string containing the name of the plot (the format is by default .png). For more options see \code{\link{scplot}}.
-#' @param cores number of cores to be used by the command. The default is one.
+#' @param cores number of cores to be used by the command. The default is one. When the weighting matrix \eqn{\mathbf{V}} is diagonal
+#' this option has no effect.
 #' @param w.bounds a \eqn{N_1\cdot T_1\times 2} matrix with the user-provided bounds on \eqn{\beta}. If \code{w.bounds} is provided, then
 #' the quantification of in-sample uncertainty is skipped. It is possible to provide only the lower bound or the upper bound
 #' by filling the other column with \code{NA}s.
@@ -86,9 +87,11 @@
 #' \widehat{\mathbf{r}})^{\prime}}. If \code{e.bounds} is provided, then
 #' the quantification of out-of-sample uncertainty is skipped. It is possible to provide only the lower bound or the upper bound
 #' by filling the other column with \code{NA}s.
-#'
+#' @param force.joint.PI.optim this option is here mostly for backward-compatibility. If FALSE (the default) it solves a separate optimization problem for each
+#' treated unit when it comes to quantify in-sample uncertainty as long as the weighting matrix \eqn{\mathbf{V}} is diagonal.
+#' If TRUE it solves a joint optimization problem for all treated units to quantify in-sample uncertainty. Both are valid approaches as
+#' we detail in the main paper (Cattaneo, Feng, Palomba, and Titiunik (2024)). The former is faster and less conservative.
 #' @param save.data a character specifying the name and the path of the saved dataframe containing the processed data used to produce the plot. 
-#'
 #' @param verbose if \code{TRUE} prints additional information in the console.
 #'
 #' @return
@@ -100,9 +103,9 @@
 #' \item{P}{a matrix whose rows are the vectors used to predict the out-of-sample series for the synthetic unit(s).}
 #' \item{Y.pre}{a matrix containing the pre-treatment outcome of the treated unit(s).}
 #' \item{Y.post}{a matrix containing the post-treatment outcome of the treated unit(s).}
-#' \item{Y.pre.agg}{a matrix containing the aggregate pre-treatment outcome of the treated unit(s). This differs from 
+#' \item{Y.pre.agg}{a matrix containing the aggregate pre-treatment outcome of the treated unit(s). This differs from
 #' Y.pre only in the case 'effect' in \code{scdataMulti()} is set to either 'unit' or 'time'.}
-#' \item{Y.post.agg}{a matrix containing the aggregate post-treatment outcome of the treated unit(s). This differs from 
+#' \item{Y.post.agg}{a matrix containing the aggregate post-treatment outcome of the treated unit(s). This differs from
 #' Y.post only in the case 'effect' in \code{scdataMulti()} is set to either 'unit' or 'time'.}
 #' \item{Y.donors}{a matrix containing the pre-treatment outcome of the control units.}
 #' \item{specs}{a list containing some specifics of the data:
@@ -207,9 +210,9 @@
 #'
 #' \item{If \code{name == "ols"}, then the problem is unconstrained and the vector of weights
 #' is estimated via ordinary least squares.}
-#' 
+#'
 #' \item{If \code{name == "L1-L2"}, then
-#' \deqn{||\mathbf{w}||_1 = 1,\:\:\: ||\mathbf{w}||_2 \leq Q,}
+#' \deqn{||\mathbf{w}||_1 = 1,\:\:\: ||\mathbf{w}||_2 \leq Q, \:\:\: w_j \geq 0,\:\: j =1,\ldots,J.}
 #' where \eqn{Q} is a tuning parameter computed as in the "ridge" case.}
 #' }}
 #'
@@ -224,7 +227,7 @@
 #' \item{if the user wants to provide their own weighting matrix, then it must use the option \code{V.mat} to input a \eqn{v\times v} positive-definite matrix, where \eqn{v} is the 
 #' number of rows of \eqn{\mathbf{B}} (or \eqn{\mathbf{C}}) after potential missing values have been removed. In case the user
 #' wants to provide their own \code{V}, we suggest to check the appropriate dimension \eqn{v} by inspecting the output
-#' of either \code{scdata} or \code{scdataMulti} and check the dimensions of \eqn{\mathbf{B}} (and \eqn{\mathbf{C}}). Note that 
+#' of either \code{scdata} or \code{scdataMulti} and check the dimensions of \eqn{\mathbf{B}} (and \eqn{\mathbf{C}}). Note that
 #' the weighting matrix could cause problems to the optimizer if not properly scaled. For example, if \eqn{\mathbf{V}} is diagonal
 #' we suggest to divide each of its entries by \eqn{\|\mathrm{diag}(\mathbf{V})\|_1}.}
 #' }}
@@ -232,7 +235,7 @@
 #'
 #' \item{\strong{Regularization.} \code{rho} is estimated through the formula
 #' \deqn{\varrho = \mathcal{C}\frac{\log (T_0)^c}{T_0^{1/2}}}
-#' where \eqn{\mathcal{C} = \widehat{\sigma}_u / \min_j \widehat{\sigma}_{b_j}} if \code{rho = 'type-1'}, 
+#' where \eqn{\mathcal{C} = \widehat{\sigma}_u / \min_j \widehat{\sigma}_{b_j}} if \code{rho = 'type-1'},
 #' \eqn{\mathcal{C} = \max_{j}\widehat{\sigma}_{b_j}\widehat{\sigma}_{u} / \min_j \widehat{\sigma}_{b_j}^2} if \code{rho = 'type-2'}, and
 #' \eqn{\mathcal{C} = \max_{j}\widehat{\sigma}_{b_ju} / \min_j \widehat{\sigma}_{b_j}^2} if \code{rho = 'type-3'},
 #'
@@ -244,9 +247,9 @@
 #' First of all, estimation of the first moment of \eqn{\mathbf{u}} can be controlled through
 #' the option \code{u.missp}. When \code{u.missp = FALSE}, then \eqn{\mathbf{E}[u\: |\: \mathbf{D}_u]=0}. If instead \code{u.missp = TRUE},
 #' then \eqn{\mathbf{E}[\mathbf{u}\: |\: \mathbf{D}_u]} is estimated using a linear regression of
-#' \eqn{\widehat{\mathbf{u}}} on \eqn{\mathbf{D}_u}. The default set of variables in \eqn{\mathbf{D}_u} is composed of \eqn{\mathbf{B}}, 
-#' \eqn{\mathbf{C}} and, if required, it is augmented with lags (\code{u.lags}) and polynomials (\code{u.order}) of \eqn{\mathbf{B}}. 
-#' The option \code{u.design} allows the user to provide an ad-hoc set of variables to form \eqn{\mathbf{D}_u}. 
+#' \eqn{\widehat{\mathbf{u}}} on \eqn{\mathbf{D}_u}. The default set of variables in \eqn{\mathbf{D}_u} is composed of \eqn{\mathbf{B}},
+#' \eqn{\mathbf{C}} and, if required, it is augmented with lags (\code{u.lags}) and polynomials (\code{u.order}) of \eqn{\mathbf{B}}.
+#' The option \code{u.design} allows the user to provide an ad-hoc set of variables to form \eqn{\mathbf{D}_u}.
 #' Regarding the second moment of \eqn{\mathbf{u}}, different estimators can be chosen:
 #' HC0, HC1, HC2, HC3, and HC4 using the option \code{u.sigma}.}
 #'
@@ -259,10 +262,10 @@
 #' model; "qreg" uses conditional quantile regression of the residuals \eqn{\mathbf{e}} on \eqn{\mathbf{D}_e}.}
 #'
 #' \item{\strong{Residual Estimation Over-fitting.} To estimate conditional moments of \eqn{\mathbf{u}} and \eqn{e_t}
-#' we rely on two design matrices, \eqn{\mathbf{D}_u} and \eqn{\mathbf{D}_e} (see above). Let \eqn{d_u} and \eqn{d_e} be the number of 
+#' we rely on two design matrices, \eqn{\mathbf{D}_u} and \eqn{\mathbf{D}_e} (see above). Let \eqn{d_u} and \eqn{d_e} be the number of
 #' columns in \eqn{\mathbf{D}_u} and \eqn{\mathbf{D}_e}, respectively. Assuming no missing values and balanced features, the
 #' number of observation used to estimate moments of \eqn{\mathbf{u}} is \eqn{N_1\cdot T_0\cdot M}, whilst for moments of \eqn{e_t} is \eqn{T_0}.
-#' Our rule of thumb to avoid over-fitting is to check if \eqn{N_1\cdot T_0\cdot M \geq d_u + 10} or \eqn{T_0 \geq d_e + 10}. If the 
+#' Our rule of thumb to avoid over-fitting is to check if \eqn{N_1\cdot T_0\cdot M \geq d_u + 10} or \eqn{T_0 \geq d_e + 10}. If the
 #' former condition is not satisfied we automatically set \code{u.order = u.lags = 0}, if instead the latter is not met
 #' we automatically set \code{e.order = e.lags = 0}.}
 #'
@@ -304,7 +307,7 @@
 #' result <- scpi(df, w.constr = list(name = "simplex", Q = 1), cores = 1, sims = 10)
 #' result <- scpi(df, w.constr = list(lb = 0, dir = "==", p = "L1", Q = 1),
 #'                cores = 1, sims = 10)
-#'                            
+#'
 #' @export
 
 scpi  <- function(data,
@@ -333,6 +336,7 @@ scpi  <- function(data,
                   plot.name    = NULL,
                   w.bounds     = NULL,
                   e.bounds     = NULL,
+                  force.joint.PI.optim = FALSE,
                   save.data    = NULL,
                   verbose      = TRUE) {
 
@@ -341,10 +345,10 @@ scpi  <- function(data,
     stop("data should be the object returned by running scdata or scdata_multi!")
   }
 
-  if (methods::is(data, 'scdata') == TRUE) {
-    class.type <- 'scpi_data'
-  } else if (methods::is(data, 'scdataMulti') == TRUE) {
-    class.type <- 'scpi_data_multi'
+  if (methods::is(data, "scdata") == TRUE) {
+    class.type <- "scpi_data"
+  } else if (methods::is(data, "scdataMulti") == TRUE) {
+    class.type <- "scpi_data_multi"
   }
 
   #############################################################################
@@ -386,8 +390,9 @@ scpi  <- function(data,
   outcome.var <- sc.pred$data$specs$outcome.var           # name of outcome variable
   sc.effect   <- sc.pred$data$specs$effect                # Causal quantity of interest
   sparse.mat  <- sc.pred$data$specs$sparse.matrices       # Whether sparse matrices are involved or not
-  
-  if (class.type == 'scpi_data') {
+  V.diag      <- Matrix::isDiagonal(V)                    # Check if weighting matrix is diagonal (to be used in sims)
+
+  if (class.type == "scpi_data") {
     Jtot            <- J
     KMI             <- KM
     I               <- 1
@@ -399,7 +404,7 @@ scpi  <- function(data,
     T0              <- list(T0)
     names(T0)       <- sc.pred$data$specs$treated.units
 
-  } else if (class.type == 'scpi_data_multi') {
+  } else if (class.type == "scpi_data_multi") {
     J               <- unlist(J)
     Jtot            <- sum(J)
     KMI             <- data$specs$KMI                              # total number of covariates used for adjustment
@@ -422,10 +427,10 @@ scpi  <- function(data,
     if (is.matrix(P) == FALSE) {
       stop("The object P should be a matrix!")
     }
-    if (class.type == 'scpi_data') {
+    if (class.type == "scpi_data") {
       Prows <- T1
-      Pcols <- J+KM
-    } else if (class.type == 'scpi_data_multi') {
+      Pcols <- J + KM
+    } else if (class.type == "scpi_data_multi") {
       Prows <- T1.tot
       Pcols <- Jtot + KMI
     }
@@ -453,7 +458,7 @@ scpi  <- function(data,
     stop("The object u.sigma should be of type character!!")
 
   } else {
-    if (!(u.sigma %in% c('HC0','HC1','HC2','HC3','HC4')))  {
+    if (!(u.sigma %in% c("HC0", "HC1", "HC2", "HC3", "HC4")))  {
       stop("Supported variance estimators are 'HC0','HC1','HC2','HC3','HC4'.")
     }
   }
@@ -471,14 +476,14 @@ scpi  <- function(data,
 
     if (nrow(w.bounds) != length(Y.post.fit)) {
       stop(paste0("w.bounds should be a matrix with ", length(Y.post.fit),
-                 " rows (i.e. the number of post-intervention periods)."))
+                  " rows (i.e. the number of post-intervention periods)."))
     }
   }
 
   if (sims < 10) {
     stop("The number of simulations needs to be larger or equal than 10!")
   }
-  
+
   if (is.null(e.bounds) == FALSE) {
     if (is.matrix(e.bounds) == FALSE) {
       stop("The object e.bounds should be a matrix!")
@@ -489,17 +494,17 @@ scpi  <- function(data,
            lower bound, the second for the upper. In case you don't want to specify
            the lower or the upper bound just fill the specific column with NAs.")
     }
-    
+
     if (nrow(e.bounds) != length(Y.post.fit)) {
-      stop(paste("e.bounds should be a matrix with ", length(Y.post.fit), 
+      stop(paste("e.bounds should be a matrix with ", length(Y.post.fit),
                  " rows (i.e. the number of post-intervention periods)."))
     }
-  }  
+  }
 
   # Check rho
   if (is.null(rho) == FALSE) {
     if (is.character(rho) == TRUE) {
-      if (!(rho %in% c('type-1','type-2','type-3'))) {
+      if (!(rho %in% c("type-1", "type-2", "type-3"))) {
         stop("When not a scalar, 'rho' must be 'type-1', 'type-2', or 'type-3'.")
       }
     }
@@ -515,18 +520,20 @@ scpi  <- function(data,
     }
   } else {
     cores <- parallel::detectCores(logical = TRUE) - 1
-    warning(paste("scpi is using",cores,"cores for estimation! You can adjust the number of cores with the 'cores' option."), immediate. = TRUE, call. = FALSE)
+    warning(paste("scpi is using", cores,
+                  "cores for estimation! You can adjust the number of cores with the 'cores' option."),
+                  immediate. = TRUE, call. = FALSE)
   }
 
   if (verbose) {
-    if (class.type == 'scpi_data') {
-      constr.type <- w.constr[['name']]
-    } else if (class.type == 'scpi_data_multi') {
+    if (class.type == "scpi_data") {
+      constr.type <- w.constr[["name"]]
+    } else if (class.type == "scpi_data_multi") {
       constr.type <- w.constr[[1]]$name
     }
 
     cat("Quantifying Uncertainty\n")
-    #executionTime(T0.tot, Jtot, I, T1.tot, sims, cores, constr.type)
+    # executionTime(T0.tot, Jtot, I, T1.tot, sims, cores, constr.type)
   }
 
   # create lists of matrices
@@ -549,7 +556,7 @@ scpi  <- function(data,
     names <- strsplit(colnames(P), "\\.")
     cnames <- unlist(lapply(names, "[[", 1))
     for (tr in sc.pred$data$specs$treated.units) {
-      P.list[[tr]] <- P[, cnames == tr, drop=FALSE]
+      P.list[[tr]] <- P[, cnames == tr, drop = FALSE]
     }
   } else {
     P.list <- mat2list(P)
@@ -560,7 +567,7 @@ scpi  <- function(data,
   } else {
     Pd.list <- rep(list(NULL), I)
   }
-  
+
   V.list   <- mat2list(V)
   w.list   <- mat2list(as.matrix(w))
   res.list <- mat2list(res)
@@ -572,17 +579,17 @@ scpi  <- function(data,
     if (is.null(C) == FALSE) C.list <- lapply(C.list, as.matrix)
     P.list <- lapply(P.list, as.matrix)
   }
-  
+
   #############################################################################
   #############################################################################
   ### Estimate In-Sample Uncertainty
   #############################################################################
   #############################################################################
 
-  if (class.type == 'scpi_data') {
+  if (class.type == "scpi_data") {
     w.constr.list <- list(w.constr)
     names(w.constr.list) <- sc.pred$data$specs$treated.units
-  } else if (class.type == 'scpi_data_multi') {
+  } else if (class.type == "scpi_data_multi") {
     w.constr.list <- w.constr
   }
 
@@ -592,7 +599,7 @@ scpi  <- function(data,
 
   for (i in seq_len(I)) {
     ## Regularize W and local geometry (treated unit by treated unit)
-    loc.geom <- local.geom(w.constr.list[[i]], rho, rho.max, res.list[[i]], B.list[[i]], 
+    loc.geom <- local.geom(w.constr.list[[i]], rho, rho.max, res.list[[i]], B.list[[i]],
                            C.list[[i]], coig.data[[i]], T0.M[[i]], J[[i]], w.list[[i]],
                            verbose)
 
@@ -614,7 +621,7 @@ scpi  <- function(data,
           warning("At least one of your features is observed for less periods than the number of lags, u.lags reverted to 0.", immediate. = TRUE, call. = FALSE)
         }
         u.lags <- 0
-       }
+      }
     }
 
     ## Prepare design matrix for in-sample uncertainty
@@ -623,17 +630,17 @@ scpi  <- function(data,
                       features[[i]], feature.id, u.design, res.list[[i]])
 
     u.names <- c(u.names, colnames(obj$u.des.0))
-    
+
     u.des.0 <- Matrix::bdiag(u.des.0, obj$u.des.0)
     f.id <- c(f.id, as.factor(feature.id))
-    
+
     ## Prepare design matrices for out-of-sample uncertainty
     e.des <- e.des.prep(B.list[[i]], C.list[[i]], P.list[[i]], e.order, e.lags,
                         res.list[[i]], sc.pred, Y.d.list[[i]], out.feat[[i]],
                         features[[i]], J[[i]], index.i, loc.geom$index.w,
                         coig.data[[i]], T0[[i]][outcome.var], T1[[i]], constant[[i]], 
                         e.design, Pd.list[[i]], sc.pred$data$specs$effect, I, class.type)
- 
+
     e.res   <- c(e.res, e.des$e.res)
     e.rownames <- c(e.rownames, rownames(e.des$e.res))
     cnames <- rep(paste0(names(w.constr.list)[[i]], "."), ncol(e.des$e.des.0))
@@ -641,7 +648,7 @@ scpi  <- function(data,
 
     if (sc.pred$data$specs$effect == "time") {
       trname <- unlist(purrr::map(stringr::str_split(rownames(e.des$e.des.0)[1], "\\."), 1))
-      rnames <- paste(trname, as.character(c(1:nrow(e.des$e.des.1))), sep=".")
+      rnames <- paste(trname, as.character(c(1:nrow(e.des$e.des.1))), sep = ".")
       e1.rownames <- c(e1.rownames, rnames)
     }
 
@@ -724,21 +731,22 @@ scpi  <- function(data,
 
   # Proceed cleaning missing data in the post-treatment period
   rowKeep <- Matrix::rowSums(is.na(P)) == 0
-  P.na <- P[rowKeep, ]
+  P.na <- P[rowKeep, , drop = FALSE]
 
   #############################################################################
   ########################################################################
   ## Estimate E[u|H], V[u|H], and Sigma
   # If the model is thought to be misspecified then E[u|H] is estimated
-  
+
   if (u.missp == TRUE) {
     T.u <- nrow(u.des.0.na)
     u.des.list <- mat2list(as.matrix(u.des.0.na)) # as.matrix takes care of the sparse case
     f.id.list <- mat2list(as.matrix(f.id.na))
     u.des.0.flex <- matrix(NA, 0, 0)
     u.des.0.noflex <- matrix(NA, 0, 0)
+
     for (i in seq_len(I)) {
-      u.des.0.flex <- Matrix::bdiag(u.des.0.flex, 
+      u.des.0.flex <- Matrix::bdiag(u.des.0.flex,
                                     DUflexGet(as.matrix(u.des.list[[i]]), C.list[[i]], f.id.list[[i]], M[[i]]))
       u.des.0.noflex <- Matrix::bdiag(u.des.0.noflex, as.matrix(u.des.list[[i]]))
     }
@@ -752,23 +760,23 @@ scpi  <- function(data,
     if (u.simple) {
       if (verbose && (u.order > 0 || u.lags > 0)) {
         warning(paste0("One of u.order > 0 and u.lags > 0 was specified, however the current number of observations (",
-                      T.u, ") used to estimate conditional moments of the pseudo-residuals ",
-                      "is not larger than the number of parameters used in estimation (",ncol(u.des.0.flex),") plus 10. ",
-                      "To avoid over-fitting issues u.order and u.lags were set to 0."), immediate. = TRUE, call. = FALSE)
+                       T.u, ") used to estimate conditional moments of the pseudo-residuals ",
+                       "is not larger than the number of parameters used in estimation (", ncol(u.des.0.flex), ") plus 10. ",
+                       "To avoid over-fitting issues u.order and u.lags were set to 0."), immediate. = TRUE, call. = FALSE)
       }
       u.des.0.na <- matrix(1, nrow = T.u, 1)
       u.order <- 0
       u.lags <- 0
     } else if (u.noflex) {
       if (verbose && (u.order > 0 || u.lags > 0)) {
-        warning(paste0("The current number of observations (",T.u,") used to estimate conditional moments of the pseudo-residuals ",
-                       "is not larger than the number of parameters used in estimation (",ncol(u.des.0.flex),") plus 10 when allowing for a ",
+        warning(paste0("The current number of observations (", T.u, ") used to estimate conditional moments of the pseudo-residuals ",
+                       "is not larger than the number of parameters used in estimation (", ncol(u.des.0.flex), ") plus 10 when allowing for a ",
                        "feature specific model. To avoid over-fitting issues, the conditional moments of the pseudo-residuals are predicted with ",
                        "the same model across features."), immediate. = TRUE, call. = FALSE)
       }
       u.des.0.na <- as.matrix(u.des.0.noflex)
 
-    } else if (u.flex){
+    } else if (u.flex) {
       u.des.0.na <- as.matrix(u.des.0.flex)
 
     }
@@ -785,6 +793,7 @@ scpi  <- function(data,
 
   # Estimate degrees of freedom to be used for V[u|H] (note that w is pre-regularization)
   df <- df.EST(w.constr = w.constr.list[[1]], w = w, B = B, J = Jtot, KM = KMI)
+
   if (df >= TT) {
     df <- TT - 1
     warning(paste0("The current specification uses more degrees of freedom than observations. We suggest to increase the level of ",
@@ -802,6 +811,7 @@ scpi  <- function(data,
   # Auxiliary logical values to estimate bounds for w
   w.lb.est <- TRUE
   w.ub.est <- TRUE
+
   if (is.null(w.bounds) == FALSE) {
     if (all(is.na(w.bounds[, 1]) == FALSE)) w.lb.est <- FALSE
     if (all(is.na(w.bounds[, 2]) == FALSE)) w.ub.est <- FALSE
@@ -810,10 +820,18 @@ scpi  <- function(data,
   ## Define constrained problem to be simulated
   if (w.lb.est == TRUE || w.ub.est == TRUE) {
 
-    vsigg <- insampleUncertaintyGet(Z.na, V.na, P.na, beta, Sigma.root, J, KMI, I,
-                                    w.constr.inf[[1]], Q.star, Q2.star, lb, TT, sims, cores, verbose, 
-                                    w.lb.est, w.ub.est)
+    # if V is diagonal, we can solve the optimization problem independently for
+    # each treated unit, otherwise we solve it jointly
 
+    if (isTRUE(V.diag) && (isFALSE(force.joint.PI.optim))) {
+      vsigg <- insampleUncertaintyGetDiag(Z.na, V.na, P.na, beta, Sigma.root, J, KM, I,
+                                          w.constr.inf[[1]], Q.star, Q2.star, lb, TT, sims, cores, verbose,
+                                          w.lb.est, w.ub.est, sc.effect)
+    } else {
+      vsigg <- insampleUncertaintyGet(Z.na, V.na, P.na, beta, Sigma.root, J, KMI, I,
+                                      w.constr.inf[[1]], Q.star, Q2.star, lb, TT, sims, cores, verbose,
+                                      w.lb.est, w.ub.est)
+    }
     vsig <- vsigg[rowSums(is.na(vsigg)) < ncol(vsigg), ] # remove simulations were SOCP was not solved at any horizon
   }
 
@@ -854,7 +872,6 @@ scpi  <- function(data,
 
   ## Adjust for missing values
   P.nomiss <- Matrix::rowSums(is.na(P)) == 0   # rows of P with no missing values
-
   Wlb <- matrix(NA, nrow = nrow(P), ncol = 1)
   Wub <- matrix(NA, nrow = nrow(P), ncol = 1)
   rownames(Wlb) <- rownames(P)
@@ -913,11 +930,10 @@ scpi  <- function(data,
 
     for (i in seq_len(I)) {
 
+      aux <- matrix(1, nrow = nrow(e.des.0.na.list[[i]]), 1)
       if (sc.effect == "time") {
-        aux <- matrix(1/scale.x, nrow = nrow(e.des.0.na.list[[i]]), 1)
-        auxx <- matrix(1/scale.x, nrow = nrow(e.des.1.list[[i]]), 1)
+        auxx <- matrix(1 / scale.x, nrow = nrow(e.des.1.list[[i]]), 1)
       } else {
-        aux <- matrix(1, nrow = nrow(e.des.0.na.list[[i]]), 1)
         auxx <- matrix(1, nrow = nrow(e.des.1.list[[i]]), 1)
       }
 
@@ -936,15 +952,15 @@ scpi  <- function(data,
 
     if (verbose && (e.order > 0 || e.lags > 0)) {
       warning(paste0("One of e.order > 0 and e.lags > 0 was specified, however the current number of observations (",
-                    T.e, ") used to estimate conditional moments of the out-of-sample error",
-                    " is not larger than the number of parameters used in estimation (", params.e, ") plus 10.",
-                    " To avoid over-fitting issues e.order and e.lags were set to 0."),
-                    immediate. = TRUE, call. = FALSE)
+                     T.e, ") used to estimate conditional moments of the out-of-sample error",
+                     " is not larger than the number of parameters used in estimation (", params.e, ") plus 10.",
+                     " To avoid over-fitting issues e.order and e.lags were set to 0."),
+                     immediate. = TRUE, call. = FALSE)
     }
     e.order <- 0
     e.lags <- 0
   }
-  
+
   params.e <- ncol(e.des.0.na)
 
   e.lb.gau <- e.ub.gau <- e.lb.ls <- e.ub.ls <- e.lb.qreg <- e.ub.qreg <- e.mean <- e.var <- c()
@@ -957,6 +973,9 @@ scpi  <- function(data,
   for (i in seq_len(I)) {
     e.des.0.na.list[[i]] <- detectConstant(e.des.0.na.list[[i]])
     e.des.1.list[[i]] <- detectConstant(e.des.1.list[[i]], scale.x)
+    aux <- avoidCollin(e.des.0.na.list[[i]], e.des.1.list[[i]], scale.x)
+    e.des.0.na.list[[i]] <- aux$mat.0
+    e.des.1.list[[i]] <- aux$mat.1
 
     if (i == 1) {
       e.des.0.na <- e.des.0.na.list[[i]]
@@ -1017,7 +1036,7 @@ scpi  <- function(data,
     if (e.order == 0) {
 
       lb <- quantile(e.res.na, e.alpha / 2)
-      ub <- quantile(e.res.na, 1-e.alpha / 2)
+      ub <- quantile(e.res.na, 1 - e.alpha / 2)
 
     } else {
       pi.e   <- scpi.out(res = e.res.na, x = e.des.0.na, eval = e.des.1,
@@ -1160,11 +1179,11 @@ scpi  <- function(data,
                   est.results        = sc.pred$est.results,
                   inference.results  = inference.results)
 
-  class(result) <- 'scpi'
-  if (class.type == 'scpi_data') {
-    result$data$specs$class.type <- 'scpi_scpi'
-  } else if (class.type == 'scpi_data_multi') {
-    result$data$specs$class.type <- 'scpi_scpi_multi'
+  class(result) <- "scpi"
+  if (class.type == "scpi_data") {
+    result$data$specs$class.type <- "scpi_scpi"
+  } else if (class.type == "scpi_data_multi") {
+    result$data$specs$class.type <- "scpi_scpi_multi"
   }
 
   #############################################################################
